@@ -12,24 +12,26 @@ pub enum Error {
   NotIpv4,
 }
 
-const SLEEP_SECONDS: u32 = 60;
-
-pub async fn daemon(name: &str, port: u16) {
+pub async fn daemon(
+  name: &str,
+  port: u16,
+  duration: u32,
+  resolve: &dyn Fn(SocketAddrV4, u16, Ipv4Addr, u16),
+  reject: &dyn Fn(&dyn std::error::Error),
+) {
   let mut local_ip = Ipv4Addr::UNSPECIFIED;
   let mut pre_gateway = SocketAddrV4::new(local_ip, 0);
 
-  let seconds = Duration::from_secs(SLEEP_SECONDS.into());
+  let seconds = Duration::from_secs(duration.into());
+  let duration = duration + 9;
   let mut ext_port = port;
   loop {
-    match upnp(name, port, ext_port, SLEEP_SECONDS + 9).await {
+    match upnp(name, port, ext_port, duration).await {
       Ok((gateway, ip)) => {
         if ip != local_ip || gateway != pre_gateway {
           local_ip = ip;
           pre_gateway = gateway;
-          println!(
-            "upnp success ( {}:{} -> {}:{} )",
-            gateway, ext_port, ip, port
-          );
+          resolve(gateway, ext_port, ip, port);
         }
       }
       Err(err) => {
@@ -41,7 +43,9 @@ pub async fn daemon(name: &str, port: u16) {
             ext_port += 1;
           }
           continue;
-        };
+        } else {
+          reject(err);
+        }
       }
     }
     sleep(seconds).await;
